// ------------------------------------------
// Implementierung des Stack
// ------------------------------------------

#include <stdlib.h>				// notwendig für malloc() und free()
#include "stack.h"				// Eigene Schnittstelle einbinden für Typ- und Funktionsdefinitionen

/* Datenstruktur */

struct irgendwas {				// Konkrete Struktur der Datenhaltung	
	data_t content;				// 	Speicherung des Stack als einfach verkettete Liste
	stack_t next;				//	content <- eigentliches Datum
};						//	next	<- nächstes Listenelement



/* Implementierung der Funktionen */

int stack_push(data_t content, stack_t stack) {	// stack ist Zeiger auf Kopfelement der Liste
	
	// Speicher für ein neues Listenelement reservieren
	stack_t newElem = (stack_t) malloc(sizeof(struct irgendwas));

	if(newElem == NULL)			// Prüfung auf Erfolg der Speicherreservierung
		return 1;			// 	-> Speicherreservierungsfehler
	
	// Datum in neues Listenelement speichern
	newElem->content = content;	
	
	// Listenzeiger umsetzen
	newElem->next = stack->next;		 
	stack->next = newElem;

	return 0;
}

int stack_pop(data_t *dest, stack_t stack) {

	// Element in einen Zwischenspeicher legen
	stack_t popElem = stack->next;

	// Wenn Zwischenspeicher leer -> Stack ist leer -> kein Element zum auslesen vorhanden
	if (popElem == NULL)
		return 1;

	// Datum aus gelesenem Element an Zieladresse speichern	
	*dest = popElem->content;

	// Kopfelement auf Nachfolger des entnommenden Elements zeigen lassen
	stack->next = popElem->next;
	
	// Speicher des zu entnommenden Elements freigeben
	free(popElem);
	return 0;
}

stack_t stack_create() {
	// Speicher für das Kopfelement der Liste reservieren
	stack_t newHead = (stack_t) malloc(sizeof(struct irgendwas));
	
	if(newHead == NULL)			// Prüfung auf Erfolg der Speicherreservierung
		return NULL;			//	-> Speicherreservierungsfehler

	// Neuer Stack ist leer -> Kopf zeigt auf NULL
	newHead->next = NULL;	
	
	// Rückgabe des Kopfelements
	return newHead;
}

int stack_destroy(stack_t stack) {
	data_t dummy;
	
	// Alle noch vorhanden Elemente aus dem Stack entfernen
	while(!stack_pop(&dummy, stack));

	// Speicher des Kopfelements freigeben
	free(stack);

	return 0;
}
